/***************************************************************************
                          base_class.h  -  description
                             -------------------
    begin                : Thu May 13 2004
    copyright            : (C) 2004 by Harm Anne Schotanus
    email                : spearhead@wanadoo.nl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef BASE_CLASS_H
#define BASE_CLASS_H

#include <string>
#include "riv2ascii.h"
#include "riv_error.h"
#include "r2a_prog_error.h"


using std::string;


/**
  *@author Harm Anne Schotanus
  */

class base_class {
private:
  string description;
	unsigned min_axles;
  unsigned max_axles;
	unsigned max_year;
  unsigned min_year;

public:
	base_class() throw ();
  base_class (
    const unsigned min_axles,
    const unsigned max_axles,
  	const unsigned min_year,
    const unsigned max_year,
    const string& description
  ) throw (ill_value, inv_letter);
  virtual ~base_class() throw () {}
  const string& get_description() const throw ();
  unsigned get_max_year() const throw ();
	unsigned get_min_year() const throw ();
	unsigned get_max_axles() const throw ();
	unsigned get_min_axles() const throw ();
};


inline const string&
base_class::get_description() const throw ()
{
  return description;
}	


inline unsigned
base_class::get_max_year() const throw ()
{
  return max_year;
}


inline unsigned
base_class::get_min_year() const throw ()
{
  return min_year;
}


inline unsigned
base_class::get_max_axles() const throw ()
{
  return max_axles;
}


inline unsigned
base_class::get_min_axles() const throw ()
{
  return min_axles;
}


inline base_class::base_class() throw ()
{
  min_axles = max_axles =  max_year = min_year = 0;
}


inline base_class::base_class (
    const unsigned min_a,
    const unsigned max_a,
  	const unsigned min_y,
    const unsigned max_y,
    const string& descr
) throw (ill_value, inv_letter) : description (descr)
{
	if (max_a < min_a && max_a != 0) throw ill_value(ill_value::AXLES, min_a);
  if (min_y > max_y && max_y != 0) throw ill_value(ill_value::YEAR, min_y);
  min_axles = min_a;
  max_axles = max_a;
  min_year = min_y;
  max_year = max_y;
}
	



#endif
