/***************************************************************************
                          r2a_axles.cpp  -  description
                             -------------------
    begin                : Fri May 28 2004
    copyright            : (C) 2004 by Harm Anne Schotanus
    email                : spearhead@wanadoo.nl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <string>
#include <iostream>
#include "riv2ascii.h"
#include "riv_error.h"
#include "r2a_axles.h"
#include "r2a_html.h"
#include "r2a_notfound_error.h"
#include "r2a_file_error.h"
#include "r2a_prog_error.h"

using std::endl;
using std::cerr;


axle_set::axle_set(
  const string& s
) : axle (s), pos (0)
{
}


int
axle_set::axles_group(
  ostream& out,
  const bool html
)
{
  if(isupper(axle[pos])) {
    put_row_column(out, html);
    out << int (axle[pos] -'A' + 1) << " driven axles";
    ++pos;
    if (axle[pos] == 'o' || axle[pos] == '0') {
      out << " independently driven";
      ++pos;
    }
    if (axle[pos] == '\'') {
      out << " separated from the main frame";
      ++pos;
    }
    out << endl;
  } else if (isdigit(axle[pos])) {
    put_row_column (out, html);
    int val = axle[pos] - '0';
    if (val > 0 & val <= 9) {
      out << val << " other axles" ;
      ++pos;
    }
    else throw not_found(axle);        // XXX
    if (axle[pos] == '\'') {
      out << " separated from the main frame";
      ++pos;
    }
    out << endl;
  } else if (axle[pos] == '('){
      ++pos;
      loc_group(out, html);
      if (axle[pos] == ')') ++pos;
      else throw not_found(axle);      // XXX
      if (axle[pos] == '\'') {
        out << " separated from the main frame";
        ++pos;
      }
  } else return 0;
  return 1;
}


void
axle_set::frame_group(
  ostream& out,
  const bool html
)
{
  axles_group(out, html);
  unsigned len = axle.length() ;
  while (pos < len && axle[pos] != '+' && axle[pos] != ')')
    if (!axles_group(out, html)) break;
}


void
axle_set::loc_group(
  ostream& out,
  const bool html
)
{
  frame_group(out, html);
  if (axle[pos] == '+') {
    ++pos;
    out << " combined with" << endl;
    loc_group(out, html);
  }
  else  if (axle.length() > pos && axle[pos] == ')') ++pos;
}


void
axle_set::tender_group(
  ostream& out,
  const bool html
)
{
  for (;pos < axle.length(); pos++)
  {
    put_row_column(out, html);
    out << axle[pos];
    put_column (out, html);
    switch (axle[pos]) {
    case 'h' :
      out << ": superheated " ;
      break;
    case 'v' :
      out << ": compound  (high  and low pressure cylinder connected) ";
      break;
    case 'n' :
      out << ": not superheated ";
      break;
    case 'i' :
      out << ": internal drive ";
      break;
    case 't' :
      out <<": tender locomotive";
      break;
    case 'd' :
      pos++;
      out << axle[pos] << ": diesel drive ";
      switch (axle[pos]){
      case 'm' :
        out << " with mechanical transmission";
        break;
      case 'h' :
        out << " with hydrodynamic transmission";
        break;
      case 'e' :
        out << " with electrical transmission";
        break;
      default:
        throw not_found(axle);     // XXX
        break;
      }
      break;
    case '1' :
    case '2' :
    case '3' :
    case '4' :
    case '5' :
    case '6' :
    case '7' :
    case '8' :
    case '9' :
      out << ": with a total of " << axle[pos] << " steam cylinders";
      break;
    default  :
      //cerr << "illegal chararcter " << axle[pos] << endl;
      throw inv_letter(axle[pos]);             // XXX
      break;
    }
    out << endl;
  }
}


void
axle_set::parse(
  ostream& out,
  const bool html
)
{
  loc_group(out, html);
  tender_group(out, html);
}


void
axle_set::print(
  ostream& out,
  const bool html
)
{
  pos = 0;
  put_table p(out, html);
  parse (out, html);
}

