/***************************************************************************
                          r2a_cmdln.cpp  -  description
                             -------------------
    begin                : Mon May 31 2004
    copyright            : (C) 2004 by Harm Anne Schotanus
    email                : spearhead@wanadoo.nl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <iostream>
#include <string>
#include <fstream>
#define _GNU_SOURCE
#include <getopt.h>
#include "riv2ascii.h"
#include "riv_error.h"
#include "config.h"
#include "r2a_cmdln.h"
#include "r2a_helper.h"
#include "r2a_files.h"
#include "r2a_file_error.h"


extern char *optarg;
extern int optind, opterr, optopt;


using std::cerr;
using std::cout;
using std::endl;


void
display_version ()
{
	cout << PACKAGE << ' ' << VERSION << endl
	 	   << "Written by Harm Schotanus" << endl
			 << endl
			 << "copyright (c) 2004 Harm Schotanus" << endl
			 << endl;
}


void
usage ()
{
	cout <<" Usage: riv2ascii <options> <riv code>" << endl
	     << endl
       <<" --axles" << endl
       <<" --axle-arrangement" << endl
       <<" -A: convert axle arrangement" << endl
	     <<" -a: specify number of wheel sets" <<endl
       <<" --country-by-number" << endl
	     <<" -c: specify country code by number" << endl
       <<" --country-by-name" << endl
	     <<" -C: specify country code by name" << endl
       <<" -config-file" << endl
			 <<" --debug" << endl
			 <<" -d: set debug level" << endl
	     <<" -f: database file " << endl
       <<" --file" << endl
       <<" -F: load codes from file" << endl
       <<" --html" << endl
       <<" --help" << endl
       <<" -h: help" << endl
       <<" -H: output in HTML" << endl
       <<" --interactive" << endl 	
       <<" -i: interactive mode" << endl
       <<" --iga" << endl
       <<" -I: consult IGA database" << endl
       <<" --uic-number" << endl
       <<" --nohtml: plain text output" << endl
			 <<" --history-file" << endl
			 <<" -T: set the commandline editting history file" << endl
       <<" -u: Convert a UIC number" << endl
       <<" --version" <<  endl
       <<" -v: version" << endl
       <<" --year" << endl
	     <<" -y: specify year" << endl
       << endl
	     <<" Report bugs to: riv2ascii@xs4all.nl" << endl
	     << endl;
}


namespace cmd_line_options {


const struct option long_options[] = {
  {"axles",1,0,'a'},
  {"axle-arrangement",1,0,'A'},
  {"country-by-number",1,0,'c'},
  {"country-by-name",1,0,'C'},
  {"config-file",1,0,'f'},
  {"debug",1,0,'d'},
  {"file",1,0,'F'},
  {"help",0,0,'h'},
  {"html",2,0,'H'},
  {"interactive",0,0,'i'},
  {"iga",0,0,'I'},
  {"nohtml",0,0,'n'},
  {"print",0,0,'p'},
  {"type",1,0,'t'},
  {"uic-number",1,0,'u'},
  {"version",0,0,'v'},
  {"year",1,0,'y'},
  {"history-file",1,0,'T'},
  {NULL,0,0,0}
};


const char* command_str = "d:hy:a:C:c:f:F:H::ivA:I";


} //namespace


void
parse_options (
  const int argc,
  char** argv,
  optionsT& options
)
{
  int c = 0, oidx = 0;
  opterr = 0;   // no error to stderr from getopt
  while (1) {
    c = getopt_long (argc, argv, cmd_line_options::command_str, cmd_line_options::long_options, &oidx);
    if (c == -1) break;
    switch (c) {
    case 0:
      break;
    case ':' : // missing argument to option
      switch_arg_error (c);             // XXX
      break;
    default :
    case '?' :
      switch_error(c); // XXX
      break;
    case 'h' :
  	  usage();
			exit (0);	
			break;
		case 'v' :
			display_version();
			exit (0);
			break;
    case 'd' :
      options.debug = atoi (optarg);
      break;
    case 'A' :
      options.axles_arr = optarg;
      options.mode = AXLES;
      break;
		case 'y' :
			options.year = atoi (optarg);
			break;
		case 'a' :
			options.axles = atoi (optarg);	
			break;
		case 'c' :
			options.country = atoi (optarg);
			break;
		case 'C' :
			options.country = 0;
			options.find_cnt = true;
			options.company_name = optarg;
	    break;
		case 'f' :
      try {
  			read_file (optarg, options);
        ++options.s_file;
      }
      catch (ill_file fn) {
        cerr << fn;
      }
			break;
    case 'F' :
      options.file = optarg;
      options.mode = FILE_MODE;
      break;
    case 'T' :
      options.history_file = optarg;
      break;
		case 'p' :
			options.mode = PRINT;
			break;
		case 't' :
			options.type = maketype(*optarg);
			break;
		case 'H' :
      options.html = (optarg == NULL || *optarg != '-') ;
			break;	
    case 'n' :
      options.html = false;
      break;
  	case 'i' :
			options.mode = INTERACTIVE;
			break;
    case 'u' :
      options.uic_code = optarg;
      options.mode = NUMBER;
      break;
    case 'I' :
      options.mode = IGA;
      break;
		}
  }
  if (optind < argc) {
	 	options.riv_text = argv[optind];
    if (options.mode == NONE) options.mode = MATCH;
  }
  if (options.mode == NONE) options.mode = INTERACTIVE;
}


void
give_help()
{
  cout <<
    "<string>          Match the given code to the database for the set mode" << endl <<
    "@<string>         Matches the RIV codes databases" << endl <<
    "%<string>         Matches the IGA database by string" << endl <<
    "$<string>         Matches the wheel arrangements for locomotives" << endl <<
    "#<string>         Matches UIC number codes" << endl <<
    ":<string>         Execute the command" << endl <<
    endl <<
    "Commands: " << endl <<
    "  :axles=<num>    set match for axles to <num>" << endl <<
    "  :clear          clear all databases" << endl <<
    "  :clear=<C|c>    clear the Capitals and small letters database or countrycode database" << endl <<
    "  :country=<num   set match for country code to <num>" << endl <<
    "  :country=<str>  set match for country code to name <str>" << endl <<
    "  :doiga=<1|0>    show IGA databse on RIV code matches" << endl <<
    "  :help           show this help screen" << endl <<
    "  :html=<1|0>     switch ouput to html" << endl <<
    "  :load=<str>     load file <str>" << endl <<
    "  :mode=<r|i|w|u> set the mode for the default lookup action" << endl <<
    "  :print          show the entire database" << endl <<
    "  :read=<str>     parse condes from file <str>" << endl <<
    "  :quit           end the programme" << endl <<
    "  :type=<a|u|s|0> set match for wagon type to articulated, wagon unit, or single unit" << endl <<
    "  :version        show version number" << endl <<
    "  :year=<n>       set match for year to <n>" << endl <<
    endl;
}


