/***************************************************************************
                          r2a_country_list.cpp  -  description
                             -------------------
    begin                : Sun Jun 6 2004
    copyright            : (C) 2004 by Harm Anne Schotanus
    email                : spearhead@wanadoo.nl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include <iostream>
#include <list>
#include <string>
#include <algorithm>
#include <iterator>
#include "riv2ascii.h"
#include "r2a_helper.h"
#include "riv_error.h"
#include "r2a_country.h"
#include "r2a_country_list.h"
#include "r2a_html.h"
#include "r2a_notfound_error.h"


void
country_code_list::add(
  const handleT <country_code>& c
)
{
	unsigned nr = c->get_code();
  country_code_iter c_end = c_list.end();
	for (country_code_iter iter = c_list.begin(); iter != c_end; ++iter) {
		if ((*iter)->get_code () >= nr) {
			c_list.insert (iter, c);
      return;
		}
	}
	c_list.push_back(c);
}


namespace {


class cc_finder {
private:
  const string& cmp;
public:
  cc_finder(const string& c) : cmp(c) {}
  bool operator() (const handleT<country_code>& s) const {return !cmp.compare (s->get_abbrev());}
};


} // namespace


handleT <country_code>
country_code_list::find (
  const string& cmp
) const
{
  country_code_citer c = find_if (c_list.begin(), c_list.end(), cc_finder(cmp));
  if (c == c_list.end()) throw country_not_found(cmp);
  return *c;
}


handleT <country_code>
country_code_list::find (
  const unsigned cd
)  const
{
  country_code_citer c = find_if (c_list.begin(), c_list.end(), finder<country_code>(cd));
  if (c == c_list.end()) throw country_not_found(cd);
  return *c;
}


void
country_code_list::print(
  ostream& out,
  const bool html
) const
{
	put_table p(out, html);
  for_each(c_list.begin(), c_list.end(), printer<country_code>(out, html));
}	

