/***************************************************************************
                          r2a_fle_error.h  -  description
                             -------------------
    begin                : Tue Aug 3 2004
    copyright            : (C) 2004 by Harm Anne Schotanus
    email                : spearhead@wanadoo.nl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef R2A_FILE_ERROR_H
#define R2A_FILE_ERROR_H

#include <string>
#include <sstream>
#include "r2a_helper.h"
#include "r2a_cmdln.h"
#include "riv2ascii.h"
#include "riv_error.h"


/**
  *@author Harm Anne Schotanus
  */


using std::string;
using std::endl;
using std::ostringstream;
using std::cerr;


class file_error: public riv_error {
private:
public:
  virtual string operator() () const  {return string();}
	file_error() {}
	virtual ~file_error() {}
  virtual void print(ostream& out) const = 0;
};



class parse_error : public file_error {
private:
  string str;
  unsigned line;
public:
  parse_error (const string& s, const unsigned l=0) : str(s), line(l){}
  virtual ~parse_error () {}
  virtual string operator() () const {return str;}
  virtual void print(ostream& out) const;
  unsigned get_line() const {return line;}
};


inline ostream&
operator<<(ostream& out, const parse_error& l)
{
  out << "parse error";
  if (l.get_line() > 0) out << " at line " << l.get_line();
  out << ": " << l();
  return out;
}


inline void
parse_error::print(ostream& out) const{ out << *this;}


class ill_file : public file_error {
private:
  string filename;

public:
  virtual ~ill_file () {}
  ill_file(): filename() {}
  ill_file(const string& fn): filename (fn) { }
  ill_file (const char* fn) { if (fn != NULL) filename = fn; else filename = string();}
  virtual string operator() () const { return filename;}
  virtual void print(ostream& out) const;
};


inline ostream&
operator<<(ostream& out, const ill_file& l)
{
  out << "File not found: " << l() << endl;
  return out;
}


inline void
ill_file::print(ostream& out) const
{
  out << *this;
}


#endif
