/***************************************************************************
                          r2_helper.cpp  -  description
                             -------------------
    begin                : Mon May 31 2004
    copyright            : (C) 2004 by Harm Anne Schotanus
    email                : spearhead@wanadoo.nl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <string>
#include <iostream>
#include <algorithm>
#include <cctype>
#include <iterator>
#include "riv2ascii.h"
#include "r2a_helper.h"
#include "riv_error.h"
#include "r2a_country_list.h"
#include "r2a_file_error.h"
#include "r2a_notfound_error.h"
#include "r2a_prog_error.h"


using std::min;


namespace spaces_string {
  const string spaces = "\t\f\n\r\v ";
}


void
trim_spaces (
  string& s
)
{
  string::size_type i = s.find_first_not_of (spaces_string::spaces);
  if (i != string::npos) s = s.erase (0, i);
}


void
trim_end_spaces (
  string& s
)
{
  string::size_type i = s.find_last_not_of (spaces_string::spaces);
  if (i != string::npos) s = s.erase (i + 1, string::npos);
}


string::size_type
skip_spaces (
  const string& s,
  const string::size_type bidx,
  const string::size_type eidx
)
{
  string::size_type i = s.find_first_not_of (spaces_string::spaces.c_str(), bidx);
  return i > eidx ? eidx : i;
}


unsigned
convert_num (
  const string& s,
  const unsigned offset
)
{
  Assert<ill_value> (s.length() >= offset, ill_value::INTERNAL);
  return strtoul (s.c_str() + offset, NULL, 10);
}


unsigned
convert_num (
  const string& s1,
  bool& entire,
  const unsigned offset,
  string::size_type len
)
{
  Assert<ill_value> (s1.length() >= offset, ill_value::INTERNAL);
  string s = s1.substr(offset, len);
  char *p, *q= const_cast<char*>(s.c_str());
  unsigned long l = strtoul (q , &p, 10);
  entire = (len == string::npos)? (*p == '\0') : (unsigned (p - q) == len);
  return l;
}


unsigned
convert_num (
  const string& s,
  const unsigned offset,
  const unsigned len
)
{
  Assert<ill_value> (s.length() + len >= offset, ill_value::INTERNAL);
  string k = s.substr(offset, len);
  return strtoul (k.c_str(), NULL, 10);
}


typeT
maketype(
  const char t
)
{
	switch (t) {
		case 'a': return articulated;
		case 's': return single;
		case 'u': return unit;
		default : return unknown;
	}
}


namespace bool_strings {


const string TRUESTR = "true";
const string FALSESTR = "false";


} // namespace


unsigned
STR_EL(
  const string& s,
  const unsigned i,
  const unsigned j)
{
  if (i > j) throw ill_value(ill_value::CODE, i);
  return convert_num (s.substr(i, (j - i) + 1));
}


bool parse_bool(
  const string& s,
  const string::size_type pos,
  const string::size_type n
)
{
  if (string_compare (s, bool_strings::TRUESTR, pos, n) == 0) return true;
  else if (string_compare (s, bool_strings::FALSESTR, pos, n) == 0) return false;
  else return convert_num(s, pos);
}


namespace wagon_type_strings {


const string ARTICULATEDSTR = "articulated";
const string UNITSTR = "unit";
const string SINGLESTR = "single";
const string NONESTR = "none";


} // namespace


typeT
maketype(
  const string& s
)
{
	if (!string_compare (s, wagon_type_strings::ARTICULATEDSTR)) return articulated;
  else if (!string_compare (s, wagon_type_strings::UNITSTR)) return unit;
  else if (!string_compare (s, wagon_type_strings::SINGLESTR)) return single;
  else if (!string_compare (s, wagon_type_strings::NONESTR)) return unknown;
  else if (convert_num(s, 0) == 0) return unknown;
  else throw parse_error(s);    // XXX
}


string
type2str (
  const typeT t
)
{
	switch (t) {
		case articulated: return string ("articulated wagon");
		case single: return string ("single unit");
		case unit: return string ("wagon unit");
		case unknown: /* fall-through */
		default : return string ("unknown");
	}
}


void
write_year (
  ostream& out,
  const unsigned mn,
  const unsigned mx,
  const string& before,
  const string& after
)
{
  print_range(out, mn, mx, false, " - ", "", "until ", " and on", before, after);
}


void
write_axles (
  ostream& out,
  const unsigned mn,
  const unsigned mx,
  const string& before,
  const string& after
)
{
  print_range (out, mn, mx, false, " - ", " wheel sets", "less than ", " or more", before, after);
}


void
print_range (
  ostream& out,
  const unsigned mn,
  const unsigned mx,
  const bool print_nil,
  const string& infix,
  const string& postfix,
  const string& lessthan,
  const string& morethan,
  const string& before,
  const string& after
)
{
  out << before;
  if ((0 == mx && mn == 0) && !print_nil) return;
  if (mn == mx) out << mn;
  else if (mn == 0) out << lessthan << mx;
  else if (mx == 0) out << mn << morethan;
  else if (mn < mx) out << mn << infix << mx;
  out << postfix << after;
}


extern country_code_list country_codes;


unsigned
convert_country (
  const string& s
)
{
  try {
    return country_codes.find_country(s);
  }
  catch (country_not_found) {
    return convert_num(s);
  }
}


string& remove(string& s, const string& r)
{
  string::size_type j = r.length();
  for(string::size_type i = 0; ; ++i) {
    i = s.find(r, i);
    if (i == string::npos) return s;
    s.erase (i, j);
  }
  return s;
}


string& remove(string& s, const char l)
{
  for(string::size_type i = 0; ; ++i) {
    i = s.find(l, i);
    if (i == string::npos) return s;
    s.erase (i, 1);
  }
  return s;
}


string& remove_all_of(string& s, const string& r)
{
  for(string::size_type i = 0; ; ++i) {
    i = s.find_first_of (r, i);
    if (i == string::npos) return s;
    s.erase (i, 1);
  }
  return s;
}

