/***************************************************************************
                          r2a_helper.h  -  description
                             -------------------
    begin                : Sat Jun 26 2004
    copyright            : (C) 2004 by Harm Anne Schotanus
    email                : spearhead@wanadoo.nl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef R2A_HELPER_H
#define R2A_HELPER_H

#include <string>
#include <iostream>
#include <sstream>
#include <riv2ascii.h>
#include <r2a_handle.h>


using std::string;
using std::ostream;
using std::ostringstream;


void
trim_spaces (
  string& s
);


void
trim_end_spaces (
  string& s
);


string::size_type
skip_spaces (
  const string& s,
  const string::size_type bidx=0,
  const string::size_type eidx=string::npos
);


unsigned
convert_num (
  const string& s,
  const unsigned offset = 0
);


unsigned
convert_num (
  const string& s,
  const unsigned offset,
  const unsigned len
);


unsigned
convert_num (
  const string& s,
  bool& entire_string,
  const unsigned offset = 0,
  string::size_type eidx = string::npos
);


typeT
maketype(
  const char t
);


typeT
maketype(
  const string& s
);

string
type2str (
  const typeT t
);


void
write_year (
  ostream& out,
  const unsigned mn,
  const unsigned mx,
  const string& before="",
  const string& after=""
);


void
write_axles (
  ostream& out,
  const unsigned mn,
  const unsigned mx,
  const string& before="",
  const string& after=""

);


void
print_range (
  ostream& out,
  const unsigned mn,
  const unsigned mx,
  const bool print_nil = false,
  const string& infix = " - ",
  const string& postfix = "",
  const string& lessthan = "-",
  const string& morethan = "+",
  const string& before = "",
  const string& after = ""
);


bool
parse_bool(
  const string& s,
  const string::size_type pos = 0,
  const string::size_type n = string::npos
);


unsigned
convert_country (
  const string& s
);


template <class T>
class finder
{
private:
  unsigned cd;
public:
  finder(const unsigned c) : cd(c) {}
  bool operator() (const handleT<T>& s) const { return s->match(cd);}
};


template<class T>
class printer
{
private:
  ostream& o;
  bool h;
public:
  printer(ostream& out, const bool html=false) : o(out), h(html) {}
  void operator() (const handleT<T>& s) const { print(o, *s, h); }
};


template <class T>
inline bool
between(
  const T y,
  const T mn,
  const T mx)
{
	return ((y == 0) || (((mn == 0) || (y >= mn)) && ((mx == 0) || (y <= mx))));
}


inline int
string_compare (
  const string& s1,
  const string& s2,
  string::size_type pos=0,
  string::size_type n=string::npos
)
{
  #ifdef __NO_STD_STL_STRING_COMPARE__
  return s1.compare(s2, pos, n);   // fix for old style STL implementation of G++ on RH 7.x
  #else
  return s1.compare(pos, n, s2);   // good STL implementation
  #endif
}


string& remove(string& s, const string& r);
string& remove(string& s, const char l);
string& remove_all_of(string& s, const string& r);


unsigned
STR_EL(
  const string& s,
  const unsigned i,
  const unsigned j
);


template <class T>
string to_string (const T x)
{
  ostringstream s;
  s << x;
  return s.str();
}


inline bool
is_comment (const char l)
{
  return l == comment_char;
}


#endif


