/***************************************************************************
                          r2a_iga.cpp  -  description
                             -------------------
    begin                : Sun Jul 4 2004
    copyright            : (C) 2004 by Harm Anne Schotanus
    email                : spearhead@wanadoo.nl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "riv_error.h"
#include "r2a_helper.h"
#include "r2a_iga.h"
#include "r2a_html.h"
#include "r2a_file_error.h"

using std::endl;


iga::iga()
{
}


iga::~iga()
{
}


/*
  1		  1		Frste bogstav af <10> 	(First letter of <10>/Erster Buchstabe der <10>)
  2 -   9	  7 +1	<13>	Laveste nr   (ERSTER WAGENNR/FIRST WAGGON NR)
 10 -  21	 12		<10>	UICLTR           (UIC SOLL-GATTUNG/UIC CODE)
 22 -  27	  6		<12>	REGIME  [21_77_] (AUSTAUCHCODE/REGIME CODE)
 28 -  35	  7 +1	<48>	BANE           (BAHNGESELLSCHAFT/RAILWAY COMPANY)
 36 -  43	  8		<14>	Hjeste nr (LETZTER WAGENNUMMER/LAST WAGGON NR)
 44 -  55	 12		<15>	LITRA    (GATTUNG/CODE)
 56 -  65	 10		<16>	TYPE     (WAGENTYP/WAGGONTYPE)
 66 -  73	  8		<17>	Laveste EX-NR (EX-NUMMER/FORMER NR)
 74 -  81	  8		<18>	Hjeste EX-NR
 82 -  93	 12		<19>	EX-LITRA (EX-GATTUNG/FORMER CODE)
 94 - 101	  6 +2	<45>	ANTAL    (WAGENBESTAND/NUMBERS)
102 - 142	 40 +1	<50>	BEMRKNINGER  (BEMERKUNGEN/REMARKS)
143 - 147	  5		<20>	LOP MIN
148 - 152	  5		<21>	LOP MAX  (LP/LENGTH OVER BUFFERS)
153 - 157	  5		<22>	AXT/BCA  (ACHSSTAND/AXLE WIDTH)
158 - 161	  4		<23>	BOGIEAXT (ACHSSTAND DREHGEST/AXLE WIDTH BOGIE)
162 - 166	  5		<24>	INDVLGD  (LADELNGE/FLOOR LENGTH)
167 - 170	  4		<25>	INDVBRD  (LADEBREITE/FLOOR WIDTH)
171 - 174	  4		<26>	BUNDFL   (LADEFLCHE/FLOOR SIZE)
175 - 178	  4		<27>	IHJDSIDE (LADEHHE SEITE/HEIGTH INSIDE SIDE)
179 - 182	  4		<28>	IHJDMIDT (LADEHHE MITTE/HEIGTH INSIDE CENTER)
183 - 187	  5		<29>	RUMFANG  (LADERAUM/VOLUME)
188 - 192	  5		<30>	DBR/SSH  (TRBREITE/WIDTH OF DOOR)
193 - 196	  4		<31>	DHJ/ESH  (TRHHE/HIGHTH OF DOOR)
197 - 200	  4		<32>	GULVHJD  (BODENHHE/FLOOR HIGHTH)
201 - 205	  5		<34>	TARAVGT (EIGENGEWICHT/TARE)
206 - 211	  5 +1	<35>	MAX LAST (LADUNG/LOAD)
212 - 215	  4		<36>	V/KMH
216 - 219	  4		<37>	MAX KMH
220 - 223	  4		<38>	V/LAST
224 - 225	  2		<39>	120/00t
226 - 229	  4		<40>	BR AUT
230 - 233	  4		<41>	BR MAN
234 - 243	 10		<11>	UIC TYPE
244 - 249	  5 +1	<43>	BYGGER (BAUJAHR/YEAR BUILT)
250 - 252	  2 +1	<44>	TILGANG (ZUGANG/FIRST YEAR OF USE)
253 - 254	  2		<46>	SIDST SET (ZULETZT GESEHEN/LAST YEAR SPOTTED)
255 - 256	  2		0D 0A   CR+LF
*/


void iga::parse_iga(const string& s)
{
  if (s.length() < 254)  throw parse_error(s);
  letter = s[0];
  if (!isupper(letter)) throw parse_error(s);
  first_nr = convert_num (s, 1, 8);
  uic_code = s.substr (9, 12);
  trim_end_spaces(uic_code);
  regime_code = s.substr (21, 6);
  regime = convert_num (regime_code,0,3);
  country_code = convert_num (regime_code, 3,3);
  railway_company = s.substr (27, 7);
  last_nr = convert_num(s, 35,8);
  code = s.substr(43, 12);
  type = s.substr(55, 10);
  first_former_nr= convert_num(s, 65,8);
  last_former_nr = convert_num(s, 73,8);
  former_code = s.substr(81, 12);
  numbers = convert_num (s, 93,8);
  remarks = s.substr(101, 40);
  min_buffer_length = convert_num(s, 142,5);
  max_buffer_length = convert_num(s, 147,5);
  axle_width = convert_num(s, 152, 5);
  axle_width_bogey = convert_num(s, 157, 4);
  floor_length = convert_num(s, 161, 4);
  floor_width = convert_num(s, 166, 4);
  floor_size = convert_num(s, 170, 4);
  height_inside_side = convert_num(s, 174, 4);
  height_inside_center = convert_num(s, 178, 4);
  volume = convert_num(s, 182, 5);
  door_width = convert_num(s, 187, 5);
  door_height = convert_num(s, 192, 4);
  floor_height = convert_num(s, 196, 4);
  tare = convert_num(s, 200, 5);
  max_load = convert_num(s, 205, 6);
  speed = convert_num(s, 211, 4);
  max_distance = convert_num(s ,215,4);
  v_last = convert_num(s, 219, 4);
// XX_ = substr(223,2);
  br_aut = s.substr(225, 4);
  br_man = s.substr(229, 4);
  UIC_type = s.substr(233, 10);
  year_built = convert_num(s, 243, 6);
  first_use = convert_num(s, 249, 3);
  last_spotted = convert_num(s, 252, 2);
 }


iga::iga(const string& s)
{
  parse_iga(s);
}


void print(ostream& out, const iga& i, const bool html)
{
  put_table t(out, html);
  put_row_column(out, html);
  out << i.get_letter() << ": " << i.get_first_nr();
  if (i.get_last_nr() != 0) out << " - " << i.get_last_nr();
  put_column (out, html);
  out << ' ' << i.get_uic_code() <<endl;
  put_row_column(out, html);
  out << "Regime: " << i.get_regime() << " " << i.get_country_code() << endl;
  put_column(out, html);
  out << "Company: " << i.get_railway_company() << endl;
  put_row_column(out, html);
  out  << "Code: " << i.get_code() << endl;
  put_column(out, html);
  out << "Former code: " << i.get_former_code()  << endl;
  put_row_column(out, html);
  out  << "Type: " << i.get_type() << endl;
  put_column(out, html);
  out << "Former number: " << i.get_first_former_nr() << " - " << i.get_last_former_nr() << endl;
  put_row_column(out, html);
  out  << "Number of wagons: " << i.get_numbers() << endl;
  put_row_column(out, html);
  out << "Remarks: " << i.get_remarks() << endl;
  put_row_column(out, html);
  out << "Wagon properties:" << endl;
  put_row_column(out, html);
  out << "buffer length: " << i.get_min_buffer_length() << " - "  << i.get_max_buffer_length() << endl;
  put_column(out, html);
  out << "axle width: " << i.get_axle_width() << endl;
  put_row_column(out, html);
  out << "axle width of a bogey: " << i.get_axle_width_bogey() << endl;
  put_column(out, html);
  out << "floor: "<< i.get_floor_length() << " x " << i.get_floor_width()
    << ", size: " << i.get_floor_size() << endl;
  put_row_column(out, html);
  out << "height inside side: " << i.get_height_inside_side() << endl;
  put_column(out, html);
  out  << "height inside center: " << i.get_height_inside_center() << endl;
  put_row_column(out, html);
  out << "volume: " << i.get_volume() <<endl;
  put_column(out, html);
  out << "door: " << i.get_door_width() << " x " << i.get_door_height() << endl;
  put_row_column(out, html);
  out << "floor height: " << i.get_floor_height() << endl;
  put_column(out, html);
  out << "tare: " << i.get_tare() << endl;
  put_row_column(out, html);
  out  << "Maximun load: " << i.get_max_load() <<endl;
  put_column(out, html);
  out  << "Speed: " << i.get_speed() << endl;
  put_row_column(out, html);
  out  << "Max distance: " << i.get_max_distance() << endl;
  put_column(out, html);
  out  << "v Last: " << i.get_v_last() << endl;
  put_row_column(out, html);
  out  << "br aut: " << i.get_br_aut() << endl;
  put_column(out, html);
  out  << "br man: " << i.get_br_man() << endl;
  put_row_column(out, html);
  out << "UIC Type: " << i.get_UIC_type() <<endl;
  put_column(out, html);
  out  << "Year of build: " << i.get_year_built() << endl;
  put_row_column(out, html);
  out << "Year first used: " << i.get_first_use() << endl;
  put_column(out, html);
  out << "Year last spotted: " << i.get_last_spotted() << endl << endl;
}
