/***************************************************************************
                          r2a_iga_list.cpp  -  description
                             -------------------
    begin                : Sun Jul 11 2004
    copyright            : (C) 2004 by Harm Anne Schotanus
    email                : spearhead@wanadoo.nl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include <iostream>
#include <list>
#include <string>
#include <iterator>
#include <algorithm>
#include "r2a_iga_list.h"
#include "riv_error.h"
#include "r2a_helper.h"
#include "r2a_html.h"
#include "r2a_notfound_error.h"


void iga_list::print (ostream& out, const bool html) const
{
  put_table p(out, html);
  for_each(i_list.begin(), i_list.end(), printer<iga>(out, html));
}


class iga_finder_number {
  private:
    unsigned uic_code;
    unsigned regime;
    unsigned company_code;
    unsigned country;
    unsigned year;
    unsigned axle;
    iga_list& il;
  public:
    iga_finder_number(
      const string& u,
      const unsigned y,
      iga_list& i
    ) : year(y), il(i) {
    regime = STR_EL (u, 0, 1);
    company_code = STR_EL (u, 2, 3);
    uic_code = STR_EL (u, 4, 10);
    }
    void operator() (const handleT <iga>& t) {
      int l =  (regime == t->get_regime()) &&
       (company_code == t->get_country_code()) &&
       ((uic_code >= t->get_first_nr()) && (uic_code <= t->get_last_nr())) &&
       (between (year, t->get_year_built(), t->get_last_spotted()));
      if (l) il.add(t);
    }
};


class iga_finder_name {
  private:
    const string& uic_code;
    unsigned country;
    unsigned year;
    unsigned axle;
    iga_list& il;
  public:
    iga_finder_name(
      const string& u,
      const unsigned c,
      const unsigned y,
      iga_list& i
    ) : uic_code(u), country(c), year(y), il(i) {}
    void operator() (const handleT <iga>& t) {
      int l = (country == 0 || country == t->get_country_code()) &&
        (t->get_uic_code() == uic_code) &&
        (between (year, t->get_year_built(), t->get_last_spotted()));
      if (l) il.add(t);
    }
};


void
iga_list::find_name(
  const string& uic_code,
  const unsigned country,
  const unsigned year,
  iga_list& il
) const
{
  iga_finder_name f(uic_code, country, year, il);
  for_each(i_list.begin(), i_list.end(), f);
  if (il.empty()) throw not_found(uic_code);
}


void
iga_list::find_number(
  const string& uic_number,
  const unsigned year,
  iga_list& il
) const
{
  iga_finder_number f(uic_number, year, il);
  for_each(i_list.begin(), i_list.end(), f);
  if (il.empty()) throw not_found(uic_number);
}


void
iga_list::find(
  const string& uic_code,
  const unsigned country,
  const unsigned year,
  iga_list& il
) const
{
  if (isdigit (uic_code[0])) find_number(uic_code, year, il);
  else if (isupper (uic_code[0])) find_name(uic_code, country, year, il);
  else throw not_found(uic_code);
}

