/***************************************************************************
                          r2a_main.h  -  description
                             -------------------
    begin                : Fri Apr 30 2004
    copyright            : (C) 2004 by Harm Anne Schotanus
    email                : spearhead@wanadoo.nl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef R2A_MAIN_H
#define R2A_MAIN_H


#include <string>
#include <iostream>
#include "riv2ascii.h"
#include "r2a_sub.h"
#include "base_class.h"
#include "riv_error.h"
#include "r2a_helper.h"
#include "r2a_sub_list.h"
#include "r2a_prog_error.h"

using std::string;
using std::ostream;


class main_class : public base_class {
private:
	char letter; // capital
	handleT<sub_class_list> sub_classes;

public:
  main_class() : sub_classes (handleT<sub_class_list> (new sub_class_list)) {}
  ~main_class() throw();
  main_class(
    const char l,
    const string& desc,
    const unsigned min_a=0,
		const unsigned max_a=0,
    const unsigned min_y=0,
    const unsigned max_y=0);
 friend ostream& print_sub (
    ostream& out,
    const main_class& s,
    const bool html=false
  );

	void find_match_s(
    sub_class_list& sl,
    const string& to_match,
    const char s,
    const unsigned doubled,
    const unsigned country, 	
		const unsigned year,
    const unsigned axles,
    const typeT type=unknown
  ) const;
  int add (handleT <sub_class> &p);
  void clear();
  char get_letter() const;
  bool match (
      const char l,
      const unsigned axles=0,
      const unsigned year=0
    ) const;
  friend void print (
    ostream& out,
    const main_class& m,
    const bool html=false
  );

};


inline void
main_class::clear()
{
  sub_classes->clear();
}


inline int
main_class::add (handleT <sub_class>& p)
{
  return sub_classes->add(p);
}


inline ostream&
print_sub (
    ostream& out,
    const main_class& mc,
    const bool html
  )
{
  mc.sub_classes->print(out, html);
  return out;
}


inline bool main_class::match (
    const char l,
    const unsigned axles,
    const unsigned year
  ) const
{
	return ((l == get_letter()) &&
	  between(year, get_min_year(), get_max_year()) &&
    between(axles, get_min_axles(), get_max_axles()));
}

	
inline char
main_class::get_letter() const
{
  return letter;
}


inline
main_class::main_class(
    const char l,
    const string& desc,
    const unsigned min_a,
		const unsigned max_a,
    const unsigned min_y,
    const unsigned max_y) :
     base_class(min_a, max_a, min_y, max_y, desc),
     sub_classes ( handleT<sub_class_list> (new sub_class_list))
{
  if (!isupper(l)) throw inv_letter(l);
  letter = l;
}


#endif /* __R2A_MAIN_H__ */
