/***************************************************************************
                          r2a_main_list.cpp  -  description
                             -------------------
    begin                : Sun Jun 6 2004
    copyright            : (C) 2004 by Harm Anne Schotanus
    email                : spearhead@wanadoo.nl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <iostream>
#include <list>
#include <string>
#include <iterator>
#include <algorithm>
#include "riv2ascii.h"
#include "r2a_main_list.h"
#include "r2a_helper.h"
#include "r2a_main.h"
#include "r2a_html.h"
#include "r2a_notfound_error.h"
#include "r2a_prog_error.h"


using std::cerr;
using std::endl;


class c_finder {
private:
  char letter;
  unsigned axles;
  unsigned year;
  main_class_list& ml;

public:
  c_finder (
    const char l,
    const unsigned a,
    const unsigned y,
    main_class_list& m
  ) : letter(l), axles(a), year(y), ml(m) {}
  void operator() (const handleT <main_class>& t) {
    if (t -> match(letter, axles, year)) ml.add(t);
	}
};


void
main_class_list::find_match_c(
  const string& str,
  const char s,
  const unsigned axles,
  const unsigned year,
  main_class_list& ml
) const
{
  if (!isupper(s)) throw inv_letter(s);
  c_finder cf (s, axles, year, ml);
  for_each (m_list.begin(), m_list.end(), cf);
  if (ml.empty() ) throw letter_not_found (str, s, 1, axles, year);
}


class r_printer
{
private:
  ostream& o;
  bool h;
public:
  r_printer(ostream& out, const bool html=false) : o(out), h(html) {}
  void operator() (const handleT<main_class>& s) const {
    print(o, *s, h);
    put_column(o, h);
    print_sub(o, *s, h);
  }
};


void
main_class_list::print(
  ostream& out,
  const bool html
) const
{
	put_table p(out, html);
  for_each(m_list.begin(), m_list.end(), r_printer(out, html));
}


void
main_class_list::add(
  const handleT<main_class>& m
)
{
	char l = m->get_letter();
  Assert<ill_value> (l != 0, ill_value::INTERNAL);
  main_class_iter iter, m_end = m_list.end();
	for (iter = m_list.begin(); iter != m_end; ++iter) {
    if ((*iter)->get_letter() >= l) {
			m_list.insert(iter, m);
			return;
		}
	}
	m_list.push_back(m);	
}


void
main_class_list::add_sub (
  const string& caps,
  const char letter,
  const unsigned doubled,
	const unsigned country,
  const unsigned min_year,
  const unsigned max_year,
	const unsigned min_axles,
  const unsigned max_axles,
  const typeT type,
	const string& descr,
  const unsigned line
)
{
	main_class_iter iter, i_end ;
  string::const_iterator s_end = caps.end();
  string specific;
	int cnt;
  char cap;
	for (string::const_iterator s = caps.begin(); s != s_end; ++s) {
		cnt = 0;
    cap = *s;
    if (*s == '(') {
      string::const_iterator k = s + 1;
      cap = *k;
      while (s != s_end && *s !=')' ) ++s;
      specific = string (k, s);
    } else specific = "";
    i_end = m_list.end();
    handleT <sub_class> p (new sub_class (letter, descr, specific, doubled, country, min_year,
      max_year, min_axles, max_axles, type));
		for (iter = m_list.begin(); iter != i_end; ++iter) {
			if (cap == '*' || (*iter)->get_letter() == cap ) {
				(*iter)->add(p);
				++cnt;
			}
		}
		if (cnt == 0) {
			cerr << "Error at line " << line << " " << letter << ": no match found " << cap << endl;     // XXX
			break;
		}
	}
}


main_class_list::~main_class_list() throw()
{
}


namespace {


void
write_c_entry (
  ostream& out,
  const main_class& m,
  const char c,
  unsigned axles,
  unsigned year,
  const bool html
)
{
  put_row_column(out, html);
	out << c;
  put_column(out, html);
	if (axles == 0) {
		if (!(m.get_min_axles() == 0 && m.get_max_axles() == 0)) {
      write_axles(out, m.get_min_axles(), m.get_max_axles(), " (", ")");
		}
	}
  put_column( out, html);
  if (year == 0) {
		if (!(m.get_min_year() == 0 && m.get_max_year() == 0)) {
		  write_year(out, m.get_min_year(), m.get_max_year(), " (", ")");
		}
	}
  put_column( out, html);
	out << ": " << m.get_description() << endl;
}


} //namespace


class mns_printer
{
private:
  ostream& o;
  char l;
  unsigned axles, year;
  bool h;
public:
  mns_printer(ostream& out, const char let, const unsigned a, const unsigned y, const bool html=false)
    : o(out), l(let), axles(a), year(y), h(html) {}
  void operator() (const handleT <main_class>& s) const {
    write_c_entry(o, *s, l, axles, year, h);
  }
};


void
main_class_list::write (ostream& out, const char l, const unsigned axles, const unsigned year, const bool html)
{
  for_each(m_list.begin(), m_list.end(), mns_printer(out, l, axles, year, html));
}


