/***************************************************************************
                          r2a_notfound_error.h.h  -  description
                             -------------------
    begin                : Tue Aug 3 2004
    copyright            : (C) 2004 by Harm Anne Schotanus
    email                : spearhead@wanadoo.nl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#ifndef R2A_NOTFOUND_ERROR_H
#define R2A_NOTFOUND_ERROR_H


#include <string>
#include <sstream>
#include "r2a_helper.h"
#include "r2a_cmdln.h"
#include "riv2ascii.h"
#include "riv_error.h"


/**
  *@author Harm Anne Schotanus
  */


using std::string;
using std::endl;
using std::ostringstream;
using std::cerr;


class not_found : public riv_error{
private:
  const string str;
public:
  not_found () {}
  not_found (const string&s) : str(s) {}
  virtual ~not_found (){}
  virtual string operator() ()const  {return str;};
  virtual void print(ostream& out) const;
};


inline ostream&
operator<<(ostream& out, const not_found& l)
{
  out << "Not found " << l() << endl;
  return out;
}


inline void
not_found::print(ostream& out) const
{
  out << *this;
}


class country_not_found : public not_found {
  string str;
  unsigned cc;
public:
  country_not_found (const unsigned c) : str(), cc(c) {}
  country_not_found (const string& s) : str(s), cc(0) {}
  virtual ~country_not_found (){}
  virtual string operator() ()const  {
    if (cc == 0) return str;
    else return to_string (cc);
  }
  virtual void print(ostream& out) const;
};


inline ostream&
operator<<(ostream& out, const country_not_found& l)
{
  out << "Country code for " << l() << " not found" << endl;
  return out;
}


inline void
country_not_found::print(ostream& out) const
{
  out << *this;
}


class letter_not_found: public not_found {
private :
  char letter;
  unsigned count;
  unsigned year;
  unsigned axles;
  string str;
public:
  letter_not_found (const string& s, const char l, const unsigned cnt=1, const unsigned y=0, const unsigned a=0)
   :letter (l), count (cnt), year(y), axles(a), str(s) {}
  virtual ~letter_not_found() {}
  virtual string operator() () const {
    ostringstream s;
    s << string(count, letter) ;
    if (year != 0) s << " for year " << year;
    if (axles != 0) s << " with " << axles << " wheel sets";
    return s.str();
  }
  const string& get_str() const { return str;}
  virtual void print(ostream& out) const{ out << *this;}
};


inline ostream& operator<<(ostream& out, const letter_not_found& l)
{
  out << l() << ": Not found in " << l.get_str();
  return out;
}


#endif
