/***************************************************************************
                          r2a_prog_error.h  -  description
                             -------------------
    begin                : Tue Aug 3 2004
    copyright            : (C) 2004 by Harm Anne Schotanus
    email                : spearhead@wanadoo.nl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#ifndef RIV_PROG_ERROR_H
#define RIV_PROG_ERROR_H

#include <string>
#include <sstream>
#include "r2a_helper.h"
#include "r2a_cmdln.h"
#include "riv2ascii.h"
#include "riv_error.h"


/**
  *@author Harm Anne Schotanus
  */


using std::string;
using std::endl;
using std::ostringstream;
using std::cerr;


class prog_error : public riv_error {
private:
public:
  virtual string operator() () const  {return string();}
	prog_error() {}
	virtual ~prog_error() {}
  virtual void print(ostream& out) const = 0;
};


class inv_uic_nr : public prog_error {
private:
  string str;
public:
  virtual string operator() () const  {return str;}
	inv_uic_nr(const string& s) : str(s) {}
	virtual ~inv_uic_nr() {}
  virtual void print(ostream& out) const;
};


inline ostream&
operator<<(ostream& out, const inv_uic_nr& l)
{
  out << "Invalid UIC number " << l() << endl;
  return out;
}


inline void
inv_uic_nr::print(ostream& out) const
{
  out << *this;
}


class inv_letter: public prog_error {
private :
  char letter;

public:
  inv_letter (const char l) { letter = l;}
  virtual ~inv_letter() {}
  virtual string operator() () const { return string(1, letter);}
  virtual void print(ostream& out) const;
};


inline ostream&
operator<<(ostream& out, const inv_letter& l)
{
  out << "Illegal character " << l() << endl;
  return out;
}


inline void
inv_letter::print(ostream& out) const
{
  out << *this;
}


class ill_value : public prog_error {
public:
  typedef enum {
    UNKNOWN=0,
    COUNTRY,
    YEAR,
    AXLES,
    TYPE,
    CODE,
    MODE,
    INTERNAL=255
  } errorcodeT;

private:
  errorcodeT error_code;
  unsigned value;

public:
  ill_value (const errorcodeT err, const unsigned val=0) : error_code (err), value(val) {}
  virtual ~ill_value () {}
  virtual string operator() () const {
    if (value > 0) return to_string(value);
    else return string();
  }
  unsigned get_value() const { return value;}
  virtual void print(ostream& out) const;
};


inline ostream&
operator<<(ostream& out, const ill_value& l)
{

  switch (l.get_value()){
  case ill_value::COUNTRY:
    out << "invalid country code:" << l() << endl;
    break;
  case ill_value::YEAR:
    out << "invalid year: " << l() << endl;
    break;
  case ill_value::AXLES:
    out << "invalid axles: " << l() << endl;
    break;
  case ill_value::TYPE:
    out << "invalide type: " << l() << endl;
    break;
  case ill_value::CODE:
    out << "invalid code: " << l() << endl;
    break;
  case ill_value::MODE:
    out << "invalid mode: " << l() << endl;
    break;
  case ill_value::INTERNAL:
  case ill_value::UNKNOWN:
  default:
    out << "Internal error" << endl;
  }
  return out;
}


inline void
ill_value::print(ostream& out) const
{
  out << *this;
}


#endif
