/***************************************************************************
                          r2a_regime.cpp  -  description
                             -------------------
    begin                : Sat Jun 12 2004
    copyright            : (C) 2004 by Harm Anne Schotanus
    email                : spearhead@wanadoo.nl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include <string>
#include <iostream>
#include <algorithm>
#include "riv2ascii.h"
#include "r2a_regime.h"
#include "riv_error.h"
#include "r2a_helper.h"
#include "r2a_html.h"


using std::iostream;
using std::endl;
using std::max;

/* first 2 numbers of the code */

regimeT::regimeT() throw()
{
  lownr = highnr = 0;
}


regimeT::regimeT(
  const unsigned low,
  const unsigned high
)  throw(ill_value)
{
  if (high != 0 && low > high) throw ill_value(ill_value::CODE, low);
  if (low >= 100 or high >= 100) throw ill_value (ill_value::CODE, max (high, low));
  lownr = low;
  highnr = high;
}


regimeT::~regimeT() throw()
{
};


passenger_regime::passenger_regime(
  const unsigned low,
  const unsigned high,
  const string& pl,
  const string& descr
) throw(ill_value) : regimeT (low, high), pool (pl), description (descr)
{
}


goods_regime::goods_regime(
  const unsigned low,
  const unsigned high,
  const unsigned min_a,
  const unsigned max_a,
  const string& pl,
  const string& owr
) throw(ill_value ): regimeT (low, high), pool (pl), owner (owr)
{
  if (max_a != 0 && min_a > max_a) throw ill_value (ill_value::AXLES, min_a);

  min_axles = min_a;
  max_axles = max_a;
}


loc_regime::loc_regime (
  const unsigned low,
  const unsigned  high,
  const string& descr
) throw(ill_value): regimeT (low, high), description (descr)
{
}


loc_regime::~loc_regime() throw()
{
}


passenger_regime::~passenger_regime() throw()
{
}


goods_regime::~goods_regime() throw()
{
}


void
goods_regime::print (
  ostream& out,
  const bool html
) const throw()
{
  put_row(out, html);
  put_column(out, html);
  print_range (out, get_lownr(), get_highnr(), true);
  put_column (out, html, ": ");
  print_range (out, get_min_axles(), get_max_axles());
  out << " wheel sets, ";
  put_column(out, html);
  out << get_pool();
  put_column (out, html, ", ");
  out << get_owner() << endl;
}


void
loc_regime::print (
  ostream& out,
  const bool html
) const throw()
{
  put_row(out, html);
  put_column(out, html);
  print_range (out, get_lownr(), get_highnr(), true);
  put_column (out, html, ": ");
  out << get_description() << endl;
}


void
passenger_regime::print (
  ostream& out,
  const bool html
) const throw()
{
  put_row_column(out, html);
  print_range (out, get_lownr(), get_highnr(), true);
  put_column (out, html, ": ");
  out << get_pool();
  put_column (out, html, ": ");
  out << get_description() << endl;
}


bool
regimeT::match (
  const unsigned reg
) const throw()
{
  return reg <= get_highnr() && reg >= get_lownr();
}


goods_regime::goods_regime () throw()
{
  max_axles = min_axles = 0;
}


loc_regime::loc_regime () throw()
{
}

