/***************************************************************************
                          r2a_regime.h  -  description
                             -------------------
    begin                : Sat Jun 12 2004
    copyright            : (C) 2004 by Harm Anne Schotanus
    email                : spearhead@wanadoo.nl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef R2A_REGIME_H
#define R2A_REGIME_H

#include <string>
#include <iostream>
#include "riv2ascii.h"
#include "riv_error.h"
#include "r2a_prog_error.h"

/**
  *@author Harm Anne Schotanus
  */


using std::string;
using std::ostream;


class regimeT
{
private:
   unsigned lownr;
   unsigned highnr;

public:
  regimeT () throw ();
	regimeT (const unsigned low, const unsigned  high) throw(ill_value);
	virtual ~regimeT() throw();
  unsigned get_lownr() const throw();
  unsigned get_highnr() const throw();
  virtual void print (ostream& out, const bool html = false) const throw() =0;
  bool match (const unsigned reg) const throw();
};


inline
void print (ostream& out, const regimeT& r, const bool html = false)
{
  r.print(out, html);
}


inline unsigned
regimeT::get_lownr() const throw()
{
  return lownr;
}


inline unsigned
regimeT::get_highnr() const throw()
{
  return highnr;
}


class loc_regime: public regimeT
{
private:
  string description;

public:
  loc_regime () throw();
  loc_regime (
    const unsigned low,
    const unsigned high,
    const string& descr
  ) throw(ill_value);
  ~loc_regime() throw();
  const string& get_description() const throw();
  void print (ostream& out, const bool html = false) const throw();
};


inline const string& loc_regime::get_description() const throw()
{
  return description;
}


class passenger_regime: public regimeT
{
private:
  string pool;
  string description;

public:
  passenger_regime() throw() {};
  passenger_regime(const unsigned low, const unsigned high, const string& pl, const string& descr) throw(ill_value);
  ~passenger_regime() throw();
  const string& get_pool() const throw();
  const string& get_description () const throw();
  void print (ostream& out, const bool html = false) const throw();

};


inline const string&
passenger_regime::get_pool() const throw()
{
  return pool;
}


inline const string&
passenger_regime::get_description () const throw()
{
  return description;
}


class goods_regime: public regimeT
{
private:
  string pool;
  string owner;
  unsigned min_axles;
  unsigned max_axles;

public:
  goods_regime () throw();
  goods_regime(
    const unsigned low,
    const unsigned high,
    const unsigned min_axle,
    const unsigned max_axle,
    const string& pl,
    const string& owner
  ) throw(ill_value);
  ~goods_regime() throw();
  const string& get_pool () const throw();
  const string& get_owner () const throw();
  unsigned get_min_axles() const throw();
  unsigned get_max_axles() const throw();
  void print (ostream& out, const bool html = false) const throw();
};


inline const string&
goods_regime::get_pool () const throw()
{
  return pool;
}


inline const string&
goods_regime::get_owner () const throw()
{
  return owner;
}


inline unsigned
goods_regime::get_min_axles() const throw()
{
  return min_axles;
}


inline unsigned
goods_regime::get_max_axles() const throw()
{
  return max_axles;
}


#endif
