/***************************************************************************
                          r2a_regime_list.h  -  description
                             -------------------
    begin                : Sat Jun 12 2004
    copyright            : (C) 2004 by Harm Anne Schotanus
    email                : spearhead@wanadoo.nl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef R2A_REGIME_LIST_H
#define R2A_REGIME_LIST_H


#include <list>
#include <string>
#include <iterator>
#include <iostream>
#include "riv2ascii.h"
#include "r2a_regime.h"
#include "riv_error.h"
#include "r2a_handle.h"


using std::string;
using std::list;
using std::ostream;


class regime_list {
private:
  typedef list <handleT <regimeT> > regime_listT;
  typedef regime_listT::iterator regime_iter;
  typedef regime_listT::const_iterator regime_citer;

  regime_listT r_list;
  void add (const handleT<regimeT>& r);

public:
  void clear();

  regime_list() {}
  ~regime_list();

  void print (ostream& out, const bool html=false) const;
  handleT <regimeT> find (const unsigned reg) const;
  void add_loc (
    const unsigned min_code,
    const unsigned max_code,
    const string& extra
  );
  void add_goods(
   const unsigned min_code,
   const unsigned max_code,
   const unsigned min_axles,
   const unsigned max_axles,
   const string& pool,
   const string& owner
  );
  void add_passenger (
    const unsigned min_code,
    const unsigned max_code,
    const string& pool,
    const string& extra
  );
};


inline ostream&
operator<< (
  ostream& out,
  const regime_list& c
)
{
  c.print(out);
  return out;
}


inline void
regime_list::add_goods(
  const unsigned min_code,
  const unsigned max_code,
  const unsigned min_axles,
  const unsigned max_axles,
  const string& pool,
  const string& owner
)
{
  add (handleT<regimeT> (new goods_regime(min_code, max_code, min_axles, max_axles, pool, owner)));
}


inline void
regime_list::add_loc (
  const unsigned min_code,
  const unsigned max_code,
  const string& extra
)
{
  add (handleT<regimeT> (new loc_regime( min_code, max_code, extra)));
}


inline void
regime_list::add_passenger (
  const unsigned min_code,
  const unsigned max_code,
  const string& pool,
  const string& descr
)
{
  handleT<regimeT> h (new passenger_regime(min_code, max_code, pool, descr));
  add(h);
}


inline void
regime_list::clear()
{
  r_list.clear();
}


inline void
regime_list::add(
  const handleT <regimeT>& r
)
{
  r_list.push_back((r));
}

#endif

