/*******************************************************************************
    riv2ascii
    Copyright (C) 2003  Harm Schotanus

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*******************************************************************************/

#include <string>
#include <iostream>
#include "riv2ascii.h"
#include "r2a_sub.h"
#include "r2a_helper.h"
#include "riv_error.h"
#include "r2a_html.h"


using std::endl;


namespace {


inline bool
matchtype (
  const typeT t,
  const typeT s
)
{
	return (t == unknown || s == unknown || t == s) ;
}


} // namespace


sub_class::sub_class() throw()
{
	letter = 0;
	doubled = 0;
	country = UIC;
	type = unknown;
}


sub_class::sub_class (
  const char l,
  const string& descr,
  const string& spec,
	const unsigned letter_doubled,
  const unsigned c,
  const unsigned min_y, 		
	const unsigned max_y,
  const unsigned min_a,
  const unsigned max_a,
	const typeT tp)  throw (ill_value, inv_letter)
  : base_class (min_a, max_a, min_y, max_y, descr),  specific (spec)
{
  if (!islower(l)) throw inv_letter(l) ;
	letter = l;
	doubled = letter_doubled;
	country = c;
	type = tp;
}


const bool
sub_class:: match(
  const string& to_match,
  const char l,
  const unsigned l_doubled,
	const unsigned c_code,
  const unsigned year,
  const unsigned axles,
  const typeT tp
) const
{
	if (l != letter ||
      doubled != l_doubled ||
      (country != UIC && c_code != UIC && country != c_code) ||
	    (!between(year, get_min_year(), get_max_year())) ||
      (!between(axles, get_min_axles(), get_max_axles())) ||
	    (!matchtype(tp, get_type())) ||
      (!get_specific().empty() && string_compare (to_match, get_specific(), 0, get_specific().length()))) return false;
	else return true;
}


void
print (
  ostream& out,
  const sub_class& s,
  const bool html
)
{
  put_row_column(out, html);
	for (unsigned i = 0; i <= s.get_doubled(); ++i)	out << s.get_letter();
  if (!s.get_specific().empty()) out << " [" << s.get_specific() << "] ";
	put_column(out, html);
	if (s.get_country() != 0) out << ' ' << s.get_country() ;
	put_column(out, html);
  out << ' ';
  write_year(out, s.get_min_year(), s.get_max_year());
	put_column(out, html);
  out << ' ';
  write_axles(out, s.get_min_axles(), s.get_max_axles());
	put_column(out, html);
	out << ' ' << s.get_description() << endl;
}


