/***************************************************************************
                          r2a_sub.h  -  description
                             -------------------
    begin                : Fri Apr 30 2004
    copyright            : (C) 2004 by Harm Anne Schotanus
    email                : spearhead@wanadoo.nl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#ifndef R2A_SUB_H
#define R2A_SUB_H


#include <string>
#include <iostream>
#include "riv2ascii.h"
#include "base_class.h"
#include "r2a_prog_error.h"


using std::string;
using std::ostream;


class sub_class: public base_class {
private:
	char letter; // small letter
	unsigned doubled; //
	unsigned country;
  string specific;
	typeT type;
	
public:
	sub_class ()  throw();
	sub_class (
    const char letter,
    const string& desc,
    const string& specific,
		const unsigned letter_doubled = 0,
    const unsigned country = UIC,
		const unsigned min_y=0,
    const unsigned max_y=0,
		const unsigned min_axles=0,	
    const unsigned max_axles=0,
		const typeT type=unknown
  ) throw (ill_value, inv_letter);
  ~sub_class () throw () {}
	char get_letter() const ;
	unsigned get_doubled()const ;
	unsigned get_country () const ;
  const string& get_specific() const ;
	typeT get_type () const ;
  const bool match(
    const string& to_match,
    const char letter,
    const unsigned l_doubled,
  	const unsigned country_code,
    const unsigned year,
    const unsigned axles,
    const typeT tp
  ) const ;

};


void
print (
    ostream& out,
    const sub_class& s,
    const bool html=false
);


inline char
sub_class::get_letter() const
{
  return letter;
}


inline unsigned
sub_class::get_doubled() const
{
  return doubled;
}


inline unsigned
sub_class::get_country () const
{
  return country;
}


inline const string&
sub_class::get_specific() const
{
  return specific;
}


inline typeT
sub_class::get_type () const
{
  return type;
}


inline ostream&
operator<< (
  ostream& out,
  const sub_class& c
)
{
  print(out, c, false);
  return out;
}


inline int
operator>(const sub_class& p, const sub_class& q)
{
  return p.get_letter() > q.get_letter() ||
    (q.get_letter() == p.get_letter() && p.get_doubled() > q.get_doubled());
}


inline int
operator<(const sub_class& p, const sub_class& q)
{
  return p.get_letter() < q.get_letter() ||
    (q.get_letter() == p.get_letter() && p.get_doubled() < q.get_doubled());
}


inline int
operator==(const sub_class& p, const sub_class& q)
{
  return q.get_letter() == p.get_letter() && q.get_doubled() == p.get_doubled();
}


inline int
operator!=(const sub_class& p, const sub_class& q)
{
  return !(p == q);
}
#endif /* __R2A_SUB_H__ */

