/***************************************************************************
                          r2a_sub_list.cpp  -  description
                             -------------------
    begin                : Sun Jun 6 2004
    copyright            : (C) 2004 by Harm Anne Schotanus
    email                : spearhead@wanadoo.nl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include <iostream>
#include <list>
#include <string>
#include <cctype>
#include <iterator>
#include <algorithm>
#include "r2a_helper.h"
#include "riv2ascii.h"
#include "r2a_sub.h"
#include "r2a_sub_list.h"
#include "riv_error.h"
#include "r2a_helper.h"
#include "r2a_html.h"
#include "r2a_notfound_error.h"
#include "r2a_prog_error.h"

using std::endl;


void write_sub(ostream& out, handleT <sub_class> s, const unsigned axles, const unsigned year, const unsigned country,  const typeT tp, const bool html=false)
{
  put_row_column(out, html);
	for (unsigned  j = 0; j <= s->get_doubled(); ++j) out << s->get_letter() ;
	put_column(out, html);
	if (!(s->get_min_axles() == 0 && s->get_max_axles() == 0) && axles == 0)
	 	write_axles (out, s->get_min_axles(), s->get_max_axles(), " (", ")");
  if (!(s->get_min_year() == 0 && s->get_max_year() == 0) && year == 0)
	 	write_year (out, s->get_min_year(), s->get_max_year(), " (", ")");
	if (tp == 0 && s->get_type() != unknown)
	  out << '(' << type2str(s->get_type()) << ')';
  if (country == 0 && s->get_country() != 0) out << '(' << s->get_country() << ')';
	put_line(out, html);
	out  << ": "<< s->get_description() << endl;
}


void sub_class_list::write(ostream& out,  const unsigned axles, const unsigned year, const unsigned cntry, const typeT tp, const bool html)
{
   sub_class_iter s_end = s_list.end();
   for (sub_class_iter iter = s_list.begin(); iter != s_end; ++iter)
      write_sub (out, *iter, axles, year, cntry, tp, html);
}


void
sub_class_list::print (
  ostream& out,
  const bool html
) const
{
  put_table p(out, html);
  for_each(s_list.begin(), s_list.end(), printer<sub_class >(out, html));
}


int
sub_class_list:: add (
  const handleT <sub_class>& p
)
{
  sub_class_iter s_end = s_list.end();
	for (sub_class_iter iter = s_list.begin(); iter != s_end; ++iter) {
    if (**iter > *p) {
		  s_list.insert (iter, p);
    	return 1;
		}
	}
	s_list.push_back (p);
	return 0;
}


class s_finder{
private:
  const string& to_match;
  char letter;
  unsigned doubled;
  unsigned country;
	unsigned year;
  unsigned axles;
  typeT tp;
  sub_class_list& sl;

public:
  s_finder(const string& t_m, const char l, const unsigned d, const unsigned c, const unsigned y,
  const unsigned a, const typeT t, sub_class_list& s)
    : to_match(t_m), letter(l), doubled(d), country(c), year(y), axles(a), tp(t), sl(s) {}
  ~s_finder() {}
  void operator ()(const handleT <sub_class>& s) {
 		if (s -> match(to_match, letter, doubled, country, year, axles, tp)) {
      sl.add(s);
    }
  }
};


void
sub_class_list::find_match_s(
  sub_class_list& sl,
  const string& to_match,
  const char letter,
  const unsigned doubled,
  const unsigned country, 					
	const unsigned year,
  const unsigned axles,
  const typeT tp
) const
{
	Assert<ill_value> (doubled <= 10, ill_value::INTERNAL);
	if (!islower(letter)) throw inv_letter (letter);

  s_finder sf(to_match, letter, doubled, country, year, axles, tp, sl);
  for_each(s_list.begin(), s_list.end(), sf);
	if (sl.empty()) throw letter_not_found(to_match, letter, doubled + 1, year, axles);
}
