/***************************************************************************
                          r2a_uicnr.cpp  -  description
                             -------------------
    begin                : Fri Apr 30 2004
    copyright            : (C) 2004 by Harm Anne Schotanus
    email                : spearhead@wanadoo.nl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include <iostream>
#include <iomanip>
#include <list>
#include <string>
#include <typeinfo>
#include <iterator>
#include <sstream>
#include "riv2ascii.h"
#include "r2a_helper.h"
#include "r2a_uicnr.h"
#include "r2a_country.h"
#include "riv_error.h"
#include "r2a_html.h"
#include "r2a_regime.h"
#include "r2a_regime_list.h"
#include "r2a_country.h"
#include "r2a_country_list.h"
#include "r2a_speed_heat_list.h"
#include "r2a_speed_heat.h"
#include "r2a_main_type_list.h"
#include "r2a_main_type.h"
#include "r2a_notfound_error.h"

extern regime_list regimes;
extern country_code_list country_codes;
extern speed_heat_list speeds_heats;
extern main_type_list main_types;


using std::cerr;
using std::setfill;
using std::setw;


namespace {


unsigned
checksum(
  const string& s,
  unsigned & counter
)
{
	unsigned temp = 0, value = 0, mul = 2, count = 0;
	string::const_iterator si = s.begin();
	while (si != s.end() && isdigit(*si) && count++ <= 12){
		temp = (*si++ - '0') * mul;
		value += (temp % 10) + (temp / 10);
		mul = (mul == 2) ? 1 : 2;
	}
	counter = count;
	return value;
}


inline int
make_checksum(
  const string& s
)
{
	int retval;
  unsigned count = 0;
	retval = checksum(s, count);
	if (count != 11) return -1;
	if (retval >= 0) return 10 - (retval % 10);
	else return -1;
}


inline bool
verify_checksum (
  const string& s
)
{
	int retval;
  unsigned count;
	retval = checksum(s, count);
	if (count != 12) return false;
	if (retval >= 0 ) return (retval % 10) == 0;
	else return true;
}


} // namespace


uic_number*
parse_code(
  const string& uic1
)
{
  string uic = uic1;
  remove (uic, "- ");

  unsigned l = uic.length();
	
 	if (l == 11 || l == 12)
	{
	 	unsigned regime = STR_EL (uic, 0, 1);
    handleT <regimeT> reg = regimes.find(regime);
    unsigned railway_company = STR_EL (uic, 2, 3);
    unsigned car_type = STR_EL(uic, 4, 5);
    int control = (l <= 11) ? -1 : (int) STR_EL (uic, 11, 11);
    int given_control = control;
    if (control < 0 || control > 9 || !verify_checksum(uic))
      control =  make_checksum(uic.substr(0, 11));
    if (typeid(*reg) == typeid(loc_regime)) {
      return NULL; // TODO
    } else if (typeid(*reg) == typeid (passenger_regime)) {
      unsigned speed = STR_EL(uic, 6, 7);
      unsigned serial = STR_EL(uic, 8, 10);
      return new passenger_number (regime, railway_company, car_type, speed,serial, control, given_control);
    } else if (typeid(*reg) == typeid(goods_regime)) {
      unsigned sub_type = STR_EL (uic, 6, 6);
      unsigned serial = STR_EL (uic, 7, 10);
      return new freight_number (regime, railway_company, car_type, sub_type, serial, control, given_control);
    }
	}
  throw inv_uic_nr(uic);
}


freight_number::~freight_number() throw()
{
}



const unsigned MAX_REGIME = 99;
const unsigned MAX_RW_CMP = 99;
const unsigned MAX_CAR_TYPE = 99;
const int MAX_CONTROL = 9;
const unsigned MAX_SPEED = 99;
const unsigned MAX_P_SERIAL = 999;
const unsigned MAX_F_SERIAL = 9999;
const unsigned MAX_SUBTYPE = 9;

freight_number::freight_number(
  const unsigned rg,
  const unsigned rc,
  const unsigned ct,
  const unsigned st,
  const unsigned sr,
  const int ctr,
  const int g_ctr
 ) : uic_number(rg, rc, ct, ctr, g_ctr)
{
  if (st <= MAX_SUBTYPE)	sub_type = st;
	else throw ill_value(ill_value::INTERNAL);	
  if (sr < MAX_F_SERIAL) serial = sr;
  else throw ill_value(ill_value::INTERNAL);	
}


uic_number::~uic_number()
{
}


passenger_number::~passenger_number() throw()
{
}



passenger_number::passenger_number(
  const unsigned rg,
  const unsigned rc,
  const unsigned ct,
  const unsigned sp,
  const unsigned sr,
  const int ctr,
  const int g_ctr
) : uic_number(rg, rc, ct, ctr, g_ctr)
{
  if (sp <= MAX_SPEED)	speed = sp;
	else throw ill_value(ill_value::INTERNAL);	
  if (sr <= MAX_P_SERIAL) serial = sr;
  else throw ill_value(ill_value::INTERNAL);	
}


uic_number::uic_number (
  const unsigned rg,
  const unsigned rc,
  const unsigned ct,
  const int ctr,
  const int g_ctr
)
{
	regime = railway_company = car_type = 0;
	given_control = control = -1;
	if (rg <= MAX_REGIME)	regime = rg;
	else throw ill_value(ill_value::UNKNOWN);	
	if (rc <= MAX_RW_CMP)	railway_company = rc;
	else throw ill_value(ill_value::COUNTRY, ct);
	if (ct <= MAX_CAR_TYPE) car_type = ct;
	else throw ill_value(ill_value::UNKNOWN);
  if ((ctr <= MAX_CONTROL && ctr >= 0)|| ctr == -1) control = ctr;
	else throw ill_value(ill_value::UNKNOWN);
  if ((g_ctr <= MAX_CONTROL && g_ctr >= 0) || g_ctr == -1) given_control = g_ctr;
	else throw ill_value(ill_value::UNKNOWN);
}


string
passenger_number::get_code() const  throw()
{
  using std::ostringstream;
  string s;
  s.reserve (BUFSIZE);
  ostringstream os (s);
  os << get_regime() << ' ' << get_railway_company() << ' '
     << get_car_type() << '-' << get_speed () << ' '
     << setw(3) << setfill('0') << get_serial()
     << '-' << get_control() << endl;
  return os.str();
}


string
freight_number::get_code() const throw()
{
  using std::ostringstream;
  string s;
  s.reserve (256);
  ostringstream os (s);
  os << get_regime() << endl
     << get_railway_company() << endl
     << get_car_type() << get_sub_type() << ' '
     << (get_serial() / 1000) << ' ' << setw(3) << setfill('0') << (get_serial() % 1000)
     << '-' << get_control() << endl;
  return os.str();
}


namespace {


void
print_control(
  ostream& out,
  const signed control,
  const signed g_control,
  const bool html
)
{
  out << "Control: " << control ;
  if (g_control != -1 && g_control != control)
    out << " (Given contol: " << g_control << ')';
  out << endl;
  put_line(out, html);
}


void
print_regime(
  ostream& out,
  const unsigned regime,
  const bool html
)
{
  try {
    regimeT* r = (regimes.find(regime)).get_ptr();
    out << "Regime: " <<  regime << ": ";
    if (loc_regime* l = dynamic_cast<loc_regime*> (r))
      out << l->get_description();
    else if (passenger_regime* p = dynamic_cast<passenger_regime*> (r))
      out << p->get_pool() << " " << p->get_description();
    else if (goods_regime *g = dynamic_cast<goods_regime*>(r))
      out << "Pool: " << g->get_pool() << ", Owner: " << g->get_owner()
          << " Axles: " << g->get_min_axles() << " - " << g->get_max_axles();
    out << endl;
    put_line(out, html);
  }
  catch (not_found& l)
  {
    cerr << l;   // XXX
  }
}


void
print_railway_company(
  ostream& out,
  const unsigned railway_company,
  const bool html)
{
  try {
    handleT <country_code> c = country_codes.find(railway_company);
    out << "Company: " << railway_company << ": "
      << c->get_company() << " (" << c->get_abbrev() << ')' << endl;
    put_line(out, html);
  }
  catch (not_found)
  {
    cerr << "not found" << endl;  // XXX
  }
}


void
print_car_type(
  ostream& out,
  const unsigned car_type,
  const main_type::car_typeT t,
  const bool html
)
{
  try {
    handleT <main_type> m (main_types.find(car_type, t));
    out << "Maintype: " << car_type << ": " <<  m->get_description() << endl;
  	put_line(out, html);
  }
  catch (not_found)
  {
    cerr << "Not Found" << endl; // XXX
  }
}


void
print_speed(
  ostream& out,
  const unsigned spd,
  const bool html
)
{
  try {
    handleT <speed_heat> s ( speeds_heats.find(spd));
    out << spd << ": " << "speed: " << s->get_speed() << " heating: " << s->get_heating() << endl;
    put_line(out, html);
  }
  catch (not_found)
  {
    cerr << "Not Found" << endl;   // XXX
  }
}


void
print_serial(
  ostream& out,
  const unsigned serial,
  const bool html)
{
  out << "Serial: " << serial << endl;
  put_line(out, html);
}


void
print_subtype(
  ostream& out,
  const unsigned st,
  const bool html
)
{
  out << "Subtype: " << st << endl;
  put_line(out, html);
}


} // namespace


void
uic_number::print_number(
  ostream& out,
  const bool html
) const
{
  print_regime(out, get_regime(), html);
	print_railway_company(out, get_railway_company(), html);
}


void
passenger_number::print_number(
  ostream& out,
  const bool html
) const
{
  out << passenger_number::get_code();
  uic_number::print_number(out, html);
  print_car_type(out, get_car_type(), main_type::Passenger, html);
  print_speed(out, get_speed(), html);
	print_serial(out, get_serial(), html);
  print_control (out, get_control(), get_given_control(), html);
}


void
freight_number::print_number(
  ostream& out,
  const bool html
) const
{
  out << freight_number::get_code();
  uic_number::print_number(out, html);
  print_car_type(out, get_car_type(), main_type::Freight, html);
  print_subtype(out, get_sub_type(), html);
	print_serial(out, get_serial(), html);
  print_control (out, get_control(), get_given_control(), html);
}

