/***************************************************************************
                          r2a_uicnr.h  -  description
                             -------------------
    begin                : Fri Apr 30 2004
    copyright            : (C) 2004 by Harm Anne Schotanus
    email                : spearhead@wanadoo.nl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#ifndef R2A_UICNR_H
#define R2A_UICNR_H


#include <string>
#include <iostream>
#include "riv2ascii.h"


using std::string;


class uic_number
{
private:
	unsigned regime;	// digit 1 & 2
	unsigned railway_company; // digit 3 & 4
	unsigned car_type; /// digit 5 & 6
	int control; // digit 12
  int given_control;

public:
	uic_number (
    const unsigned rg,
    const unsigned rc,
    const unsigned ct,
    const int ctr=-1,
    const int g_ctr=-1
  );
	virtual ~uic_number();
  unsigned get_car_type() const  throw(); /// digit 5 & 6	
  unsigned get_regime() const  throw();// digit 1 & 2
	unsigned get_railway_company() const  throw(); // digit 3 & 4
	int get_control() const  throw();// digit 12
  int get_given_control() const  throw();
  virtual unsigned get_serial() const = 0;
	virtual void print_number(
    ostream& out,
    const bool html
  ) const;
	virtual string get_code() const throw() =0;
};


inline unsigned
uic_number::get_car_type() const throw()
{
  return car_type;
}


inline unsigned
uic_number::get_regime() const throw()
{
  return regime;
}	


inline unsigned
uic_number::get_railway_company() const throw()
{
  return railway_company;
}


inline int
uic_number::get_control() const throw()
{
  return control;
}


inline int
uic_number::get_given_control() const throw()
{
  return given_control;
}


class freight_number : public uic_number
{
private:
   unsigned sub_type; // digit 7
   unsigned serial; // digit 8,9,10,11

public:
  ~freight_number()  throw();
  freight_number(
    const unsigned rg,
    const unsigned rc,
    const unsigned ct,
    const unsigned st,
    const unsigned sr,
    const int ctr=-1,
    const int g_ctr=-1
  );
  unsigned get_sub_type () const throw();
  virtual unsigned get_serial () const throw();
  virtual void print_number(
    ostream& out,
    const bool html
  ) const;
  virtual string get_code() const  throw();
};


inline unsigned
freight_number::get_sub_type () const  throw()
{
  return sub_type;
}


inline unsigned
freight_number::get_serial () const throw()
{
  return serial;
}


class passenger_number : public uic_number
{
private:
	unsigned speed; // digit 7 & 8
	unsigned serial; // digit 9, 10, 11

public:
  ~passenger_number()  throw();
  passenger_number(
    const unsigned rg,
    const unsigned rc,
    const unsigned ct,
    const unsigned sp,
    const unsigned sr,
    const int ctr=-1,
    const int g_ctr=-1
  );
	unsigned get_speed() const  throw(); // digit 7 & 8
	virtual unsigned get_serial() const  throw(); // digit 9, 10, 11
  void print_number(
    ostream& out,
    const bool html
  ) const;
  virtual string get_code() const throw();
};


inline unsigned
passenger_number::get_speed() const throw()
{
  return speed;
}


inline unsigned
passenger_number::get_serial() const throw()
{
  return serial;
}


uic_number*
parse_code (
  const string& s
);


class loc_number : uic_number
{
private:


public:
	loc_number (
    const unsigned rg,
    const unsigned rc,
    const unsigned ct,
    const int ctr=-1,
    const int g_ctr=-1
  ) : uic_number(rg, rc, ct, ctr, g_ctr) {}
	virtual ~loc_number();

  virtual unsigned get_serial() const {};
	virtual void print_number(
    ostream& out,
    const bool html
  ) const {}
	virtual string get_code() const throw() {};
};

#endif /* R2A_UICNR_H */

