/***************************************************************************
                          riv_error.h  -  description
                             -------------------
    begin                : Sun May 23 2004
    copyright            : (C) 2004 by Harm Anne Schotanus
    email                : spearhead@wanadoo.nl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#ifndef RIV_ERROR_H
#define RIV_ERROR_H

#include <string>
#include <sstream>
#include "r2a_helper.h"
#include "r2a_cmdln.h"
#include "riv2ascii.h"

using std::cerr;
using std::endl;

  class  riv_error {
private:
public:
  virtual string operator() () const  {return string();}
  /** No descriptions */
  void setdescription(const string& );
	riv_error() {}
	virtual ~riv_error() {}
  virtual void print(ostream& out) const = 0;
};


template <class X, class A, class V>
inline void
Assert (const A assertion, const V value)
{
#ifndef NDEBUG
  if (!assertion) throw X(value);
#endif
}


inline void
print_parse_error (
  ostream& err,
  const string file,
  const string error,
  const unsigned line,
  const string s
)
{
  err << file << ": Error " << error << " at line: " << line << " [" << s << ']' << endl;
}


inline void
print_unknown_command(
  ostream& err,
  const string s
)
{
  err << "Unknown command " << s << endl;
}


inline void
print_unknown_mode(
  ostream& err,
  const char l
)
{
  err << "Unknown mode " << l << endl;
}

inline void
print_ill_mode(
  ostream& err,
  const string s
)
{
  err << "Unable to find: " << s  << " (wrong mode?)" << endl;
}


inline void
db_line_error(const string& filename, const riv_error& err, const unsigned line)
{
  cerr << filename << ": Error at line: " << line << " [" << err() <<']' << endl;
}


inline void
switch_arg_error (const char c)
{
  cerr << "Error: switch " << c << " needs an argument" << endl << endl;
  usage();
  exit (0);
}


inline void
switch_error (const char c)
{
  cerr << "Error: Unknown switch: " << c << endl << endl;
  usage();
  exit (0);
}	


#endif /* RIV_ERROR_H */

